import java.awt.*;

/**
 * Controller fr die <CalculatorEngine>. Stellt ein Tastenfeld  zur Bedienung des Taschenrecherns
 * bereit und verarbeitet alle Eingaben auf dem Tastenfeld weiter. Hierzu wird in Abhngigkeit von
 * der jeweiligen Eingabe eine entsprechnde Nachricht an das "Model" <ce> versandt.
 *
 * Das Tastenfeld verfgt ber Zifferntasten fr die Ziffern 0-9, Operationstasten mit den
 * Operationen Addition, Subtraktion, Multiplikation, Division und Negation sowie den beiden
 * Spezialtasten "C/Clear", um den Rechner zurckzusetzen und "=/Calculate",
 * um das Ergebnis zu berechnen.
 *
 *  @author		Stefan Berner 
 *  @copyright  Forschungsgruppe Requirements Engineering,
 *              Institut fuer Informatik, Universitaet Zuerich 
 *  @history    26.03.1997 SB Erste version
 *              19.03.1998 SB PR 3 erledigt
 *              29.03.1998 SB Kommentare ergaenzt
 *              13.03.2003 SM Revision des Codes, v1.3
 *  @version	$Date: 2007/08/27 17:14:59 $, $Author: stoiber $, $Revision: 1.3 $  
 */
class CalculatorKeypad extends Panel
	{
	/* Instanzvariablen */
	
	/**
	 * "Model", welches die Berechnung durchfhrt
	 */
	CalculatorEngine ce;

	/**
	 * Zahlentasten: b0=0, b1=1 etc.
	 */
	private Button b0, b1, b2, b3, b4, b5, b6, b7, b8, b9;

	/*
	 * Dezimalpunkt
	 */
	private Button bPnt;

	/* Operationstasten */

	/**
	 * Addition
	 */
	private Button bAdd;

	/**
	 * Addition
	 */
	private Button bSub;

	/**
	 * Multiplikation
	 */
	private Button bMul;

	/**
	 * Division
	 */
	private Button bDiv;

	/**
	 * Negation
	 */
	private Button bNeg;

	/* Spezial Tasten */

	/**
	 * Calculate bzw. "="-Taste
	 */
	private Button bClc;

	/**
	 * Clear-Taste
	 */
	private Button bClr;	
	
	
	/* Konstruktoren */
	
	/**
	 * Korrekt initialisiertes Tastenfeld. Hierzu wird die Instanzvariable <ce>
	 * mit einer CalculatorEngine belegt und die Tasten des Tastenfeldes werden
	 * angelegt.
	 * 
	 * @pre [ce] darf nicht null sein
	 * 
	 */
	CalculatorKeypad( CalculatorEngine ce )
		{
		this.ce = ce;	// Eintragen der CalculatorEngine
		
		/* Basislayout ist ein Borderlayout, in der Mitte ("Center") die Zahlen
		und Operationstasten. Rechts ("East") angeordnet die Spezialtasten */
		setLayout( new BorderLayout( 4, 4 ) );
		
		/* Setzen der Spezialtasten. Hierzu wird ein Panel erzeugt, welches
		ein GridLayout mit zwei Zeilen enthlt. Dort hinein kommen die
		beiden Spezialtasten */
		Panel p1 = new Panel();
		p1.setLayout( new GridLayout( 2, 1, 4, 4 ) );
		add( "East", p1 );
		bClr = initAndAddToPanel( new Button( "C" ), Color.orange, p1 );
		bClc = initAndAddToPanel( new Button( "="), Color.green, p1 );
		
		/*Setzen der Zahlen- und Operationstasten. Hierzu wird ein Panel
		erzeugt, welche ein GridLayout mit 4 Zeilen und 4 Spalten enthlt.
		In dieses Grid kommen die 16 Zahlen und Operationstasten */
		Panel p2 = new Panel();
		p2.setLayout( new GridLayout( 4, 4, 4, 4 ) );
		add( "Center", p2 );
		
		/* hinzufgen der Tasten der 1. Zeile des Tastenfeldes */
		b1 = initAndAddToPanel( new Button( "1" ), Color.white, p2 );
		b2 = initAndAddToPanel( new Button( "2" ), Color.white, p2 );
		b3 = initAndAddToPanel( new Button( "3" ), Color.white, p2 );
		bAdd = initAndAddToPanel( new Button( Op.Add().asString()), Color.cyan, p2 );

		/* hinzufgen der Tasten der 2. Zeile des Tastenfeldes */		
		b4 = initAndAddToPanel( new Button( "4" ), Color.white, p2 );
		b5 = initAndAddToPanel( new Button( "5" ), Color.white, p2 );
		b6 = initAndAddToPanel( new Button( "6" ), Color.white, p2 );
		bSub = initAndAddToPanel( new Button( Op.Subtract().asString()), Color.cyan, p2 );
		
		/* hinzufgen der Tasten der 3. Zeile des Tastenfeldes */		
		b7 = initAndAddToPanel( new Button( "7" ), Color.white, p2 );
		b8 = initAndAddToPanel( new Button( "8" ), Color.white, p2 );
		b9 = initAndAddToPanel( new Button( "9" ), Color.white, p2 );
		bMul = initAndAddToPanel( new Button( Op.Multiply().asString()), Color.cyan, p2 );
		
		/* hinzufgen der Tasten der 4. Zeile des Tastenfeldes */		
		b0 = initAndAddToPanel( new Button( "0" ), Color.white, p2 );
		bPnt =  initAndAddToPanel( new Button( "." ), Color.white, p2 );
		bNeg = initAndAddToPanel( new Button( Op.Negate().asString()), Color.cyan, p2 );
		bDiv = initAndAddToPanel( new Button( Op.Divide().asString()), Color.cyan, p2 );
		}
	
	/* Instanzmethoden */	
	/**
	 * Behandeln der Tastendrcke und versenden von entsprechenden Nachrichten
	 * an die CalculatorEngine
	 * 
	 * @pre -
	 * @post
	 */
	public boolean action( Event e, Object arg )
		{
		/* Tasten 0 - 9 wurde gedrckt. Wenn eine solche Taste gedrckt
		wurde, dann wird das Argument <arg> in einen String bersetzt und der
		CalculatorEngine <ce> mitgeteilt welche Taste gedrckt wurde */
		if ( e.target == b1 )
			{ ce.digit( "1" ); }
		else if ( e.target == b2 )
			{ ce.digit( "2" ); }
		else if ( e.target == b3 )
			{ ce.digit( "3" ); }
		else if ( e.target == b4 )
			{ ce.digit( "4" ); }
		else if ( e.target == b5 )
			{ ce.digit( "5" ); }
		else if ( e.target == b6 )
			{ ce.digit( "6" ); }
		else if ( e.target == b7 )
			{ ce.digit( "7" ); }
		else if ( e.target == b8 )
			{ ce.digit( "8" ); }
		else if ( e.target == b9 )
			{ ce.digit( "9" ); }
		else if ( e.target == b0 )
			{ ce.digit( "0" ); }
		/* Dezimalpunkt wurde gedrckt. Entsprechende Nachricht geht an
		die CalculatorEngine <ce> */
		else if ( e.target == bPnt )
			{ ce.period(); }
		/* Operationstaste wurde gedrckt. Wird eine Operationstaste
		gedrckt, so wird ein entsprechendes Objekt der Klasse Op erzeugt
		und der CalculatorEngine <ce> mitgeteilt welche Operation
		ausgewhlt wurde */
		else if ( e.target == bNeg )
			{ ce.operation( Op.Negate() ); }
		else if ( e.target == bAdd )
			{ ce.operation( Op.Add() ); }
		else if ( e.target == bSub )
			{ ce.operation( Op.Subtract() ); }
		else if ( e.target == bMul )
			{ ce.operation( Op.Multiply() ); }
		else if ( e.target == bDiv )
			{ ce.operation( Op.Divide() ); }
		/* Spezialtaste wurde gedrckt. Entsprechende Nachricht (clear oder
		calculate) geht an die CalculatorEngine <ce> */
		else if ( e.target == bClc )
			{ ce.calculate(); }
		else if ( e.target == bClr )
			{ ce.clear(); };	
		return false;
		}
	
	/** 
     *  Hilfsmethode wird nur bei der Objektinitialisierung bentigt und
	 *  ist nicht zum externen Gebrauch bestimmt!
	 *  Setzt die Hintergrundfarbe von <aComponent> auf <backgroundColor>,
	 *  fgt <aComponent> zum Container <aContainer> hinzu und antwortet
	 *  mit <aComponent> 
     *  @pre 	aComponent != null und aContainer != null
	 */	
	private Button initAndAddToPanel( Button aButton, Color backgroundColor, Panel aPanel )
		{
		aButton.setBackground( backgroundColor );
		aPanel.add( aButton );
		return aButton;
		}		
	}